#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import IGroupsFolderPlugin, ISecurityManager, ILocalGroup
from pyams_skin.interfaces import IPageHeader, IInnerPage
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, MANAGE_SYSTEM_PERMISSION
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import DISPLAY_MODE, IDataExtractedEvent
from z3c.table.interfaces import IColumn, IValues
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.plugin.group import GroupsFolder, Group
from pyams_security.zmi.utility import SecurityManagerPluginsTable
from pyams_skin.container import ContainerView
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn
from pyams_skin.viewlet.toolbar import ToolbarMenuItem, ToolbarAction
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.registry import query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.view import AdminView
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Invalid

from pyams_security import _


#
# Groups folder views
#

@viewlet_config(name='add-groups-folder.menu', context=ISite, layer=IAdminLayer,
                view=SecurityManagerPluginsTable, manager=IToolbarAddingMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=20)
class GroupsFolderAddMenu(ToolbarMenuItem):
    """Local groups folder add menu"""

    label = _("Add local groups folder...")
    label_css_class = 'fa fa-fw fa-users'
    url = 'add-groups-folder.html'
    modal_target = True


@pagelet_config(name='add-groups-folder.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class GroupsFolderAddForm(AdminDialogAddForm):
    """Groups folder plug-in add form"""

    title = _("System security manager")
    legend = _("Add local groups folder plug-in")
    icon_css_class = 'fa fa-fw fa-users'

    fields = field.Fields(IGroupsFolderPlugin).omit('__name__', '__parent__')
    ajax_handler = 'add-groups-folder.json'
    edit_permission = None

    def create(self, data):
        return GroupsFolder()

    def add(self, plugin):
        context = query_utility(ISecurityManager)
        context[plugin.prefix] = plugin

    def nextURL(self):
        return absolute_url(self.context, self.request, 'security-manager.html')


@subscriber(IDataExtractedEvent, form_selector=GroupsFolderAddForm)
def handle_new_user_folder_data_extraction(event):
    """Handle new groups folder plug-in form data extraction"""
    manager = query_utility(ISecurityManager)
    prefix = event.data.get('prefix')
    if prefix in manager:
        event.form.widgets.errors += (Invalid(_("Specified prefix is already used!")),)


@view_config(name='add-groups-folder.json', context=ISite, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class GroupsFolderAJAXAddForm(AJAXAddForm, GroupsFolderAddForm):
    """Groups folder plug-in add form, AJAX handler"""


@pagelet_config(name='properties.html', context=IGroupsFolderPlugin, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class GroupsFolderEditForm(AdminDialogEditForm):
    """Groups folder plug-in edit form"""

    @property
    def title(self):
        return self.context.title

    legend = _("Edit local groups folder plug-in properties")
    icon_css_class = 'fa fa-fw fa-users'

    fields = field.Fields(IGroupsFolderPlugin).omit('__name__', '__parent__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(GroupsFolderEditForm, self).updateWidgets()
        self.widgets['prefix'].mode = DISPLAY_MODE


@view_config(name='properties.json', context=IGroupsFolderPlugin, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class GroupsFolderAJAXEditForm(AJAXEditForm, GroupsFolderEditForm):
    """Groups folder plug-in edit form, AJAX handler"""


#
# Groups folder search views
#

class GroupsFolderContentsTable(BaseTable):
    """Groups folder contents table"""

    id = 'groups_folder_table'
    title = _("Local groups list")
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight datatable'}


@adapter_config(context=(IGroupsFolderPlugin, IAdminLayer, GroupsFolderContentsTable), provides=IValues)
class GroupsFolderContentsValuesAdapter(ContextRequestViewAdapter):
    """Groups folder table values adapter"""

    @property
    def values(self):
        return list(self.context.values())


@adapter_config(name='name', context=(IGroupsFolderPlugin, IAdminLayer, GroupsFolderContentsTable),
                provides=IColumn)
class NameColumn(I18nColumn, GetAttrColumn):
    """Group name column"""

    _header = _("Name")
    attrName = 'title'
    weight = 10


@adapter_config(name='description', context=(IGroupsFolderPlugin, IAdminLayer, GroupsFolderContentsTable),
                provides=IColumn)
class DescriptionColumn(I18nColumn, GetAttrColumn):
    """Users registration date column"""

    _header = _("Description")
    attrName = 'description'
    weight = 20

    def getValue(self, obj):
        return super(DescriptionColumn, self).getValue(obj) or ''


@pagelet_config(name='search.html', context=IGroupsFolderPlugin, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class GroupsFolderSearchView(AdminView, ContainerView):
    """Groups folder search view"""

    table_class = GroupsFolderContentsTable

    def __init__(self, context, request):
        super(GroupsFolderSearchView, self).__init__(context, request)


@adapter_config(context=(IGroupsFolderPlugin, IPyAMSLayer, GroupsFolderSearchView), provides=IPageHeader)
class GroupsFolderSearchViewHeaderAdapter(DefaultPageHeaderAdapter):
    """Groups folder search view header adapter"""

    back_url = '#security-manager.html'
    back_target = '#content'
    icon_class = 'fa fa-fw fa-users'
    title = _("Security manager")

    @property
    def subtitle(self):
        return self.context.title


#
# Groups views
#

@viewlet_config(name='groups-folder.toolbar.adding', context=IGroupsFolderPlugin,
                view=GroupsFolderContentsTable, layer=IAdminLayer,
                manager=IWidgetTitleViewletManager, permission=MANAGE_SYSTEM_PERMISSION)
class LocalGroupAddAction(ToolbarAction):
    """Groups folder adding action"""

    label = _("Add group")
    url = 'add-group.html'
    modal_target = True


@pagelet_config(name='add-group.html', context=IGroupsFolderPlugin, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class LocalGroupAddForm(AdminDialogAddForm):
    """Local group add form"""

    @property
    def title(self):
        return self.context.title

    legend = _("Add new local group")
    icon_css_class = 'fa fa-fw fa-users'
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    fields = field.Fields(ILocalGroup).omit('__parent__', '__name__')

    ajax_handler = 'add-group.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(LocalGroupAddForm, self).updateWidgets()
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return Group()

    def update_content(self, group, data):
        group.group_id = data.get('group_id')
        group.title = data.get('title')
        group.description = data.get('description')
        group.principals = data.get('principals')

    def add(self, group):
        self.context[group.group_id] = group


@subscriber(IDataExtractedEvent, form_selector=LocalGroupAddForm)
def handle_new_user_data_extraction(event):
    """Handle new group form data extraction"""
    data = event.data
    folder = event.form.context
    if not folder.check_group_id(data.get('group_id')):
        event.form.widgets.errors += (Invalid(_("Specified group ID can't be used!")),)


@view_config(name='add-group.json', context=IGroupsFolderPlugin, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class LocalGroupAJAXAddForm(AJAXAddForm, LocalGroupAddForm):
    """Local group add form, AJAX view"""

    def get_ajax_output(self, changes):
        translate = self.request.localizer.translate
        return {'status': 'reload',
                'location': absolute_url(self.context, self.request, 'search.html'),
                'message': translate(_("Group was created successfully"))}


@pagelet_config(name='properties.html', context=ILocalGroup, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class LocalGroupEditForm(AdminDialogEditForm):
    """Local group edit form"""

    @property
    def title(self):
        return self.context.title

    legend = _("Edit group properties")
    icon_css_class = 'fa fa-fw fa-users'
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    fields = field.Fields(ILocalGroup).omit('__parent__', '__name__')

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(LocalGroupEditForm, self).updateWidgets()
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=ILocalGroup, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class LocalGroupAJAXEditForm(AJAXEditForm, LocalGroupEditForm):
    """Local group edit form, AJAX view"""

    def get_ajax_output(self, changes):
        if 'title' in changes.get(ILocalGroup, ()):
            return {'status': 'reload',
                    'location': absolute_url(self.context.__parent__, self.request, 'search.html')}
        else:
            return super(LocalGroupAJAXEditForm, self).get_ajax_output(changes)
